﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Reversi
{
    public partial class Form1 : Form
    {
        
        private const int margines = 10;
        private Color[] koloryPól = { Color.LightGray, Color.Orange, Color.YellowGreen };

        private ReversiSilnik silnik = new ReversiSilnik(1, 8, 8);
        private Button[,] planszaWidok;

        private void twórzPlanszę()
        {
            planszaWidok = new Button[8, 8];

            int bokPlanszy = Math.Min(this.Width, this.Height);
            bokPlanszy -= 2 * margines;
            int bokPola = bokPlanszy / 9;

            for(int poziomo = 0; poziomo < planszaWidok.GetLength(0); ++poziomo)
                for(int pionowo = 0; pionowo < planszaWidok.GetLength(1); ++pionowo)
                {
                    Button b = new Button();
                    b.Width = bokPola;
                    b.Height = bokPola;
                    b.Left = margines + poziomo * bokPola;
                    b.Top = margines + pionowo * bokPola;
                    b.Parent = this;
                    b.Click += polePlanszy_Click;
                    b.Tag = new Tuple<int, int>(poziomo, pionowo);
                    //b.Text = poziomo.ToString() + ", " + pionowo.ToString();
                    planszaWidok[poziomo, pionowo] = b;
                }
        }

        private void polePlanszy_Click(object sender, EventArgs e)
        {
            Tuple<int, int> pole = (sender as Button).Tag as Tuple<int, int>;
            if (silnik.PobierzStanPola(pole.Item1, pole.Item2) != 0) return;
            if (pole == null) throw new Exception("Brak danych o współrzędnych pola");
            ListBox lbRuchyGracz = (silnik.NumerGraczaWykonującegoNastępnyRuch == 1) ? lbRuchyGracz1 : lbRuchyGracz2;
            if (silnik.PołóżKamień(pole.Item1, pole.Item2) > 0)
            {
                lbRuchyGracz.Items.Add(symbolPola(pole.Item1, pole.Item2));
                (sender as Button).Enabled = false;
            }
            pokażPlanszę();

            //sytuacje specjalne
            ReversiSilnik.SytuacjaNaPlanszy sytuacjaNaPlanszy = silnik.ZbadajSytuacjęNaPlanszy();
            bool koniecGry = false;
            switch(sytuacjaNaPlanszy)
            {
                case ReversiSilnik.SytuacjaNaPlanszy.BieżącyGraczNieMożeWykonaćRuchu:
                    MessageBox.Show("Gracz " + silnik.NumerGraczaWykonującegoNastępnyRuch + " zmuszony jest do oddania ruchu");
                    silnik.Pasuj();
                    pokażPlanszę();
                    break;
                case ReversiSilnik.SytuacjaNaPlanszy.ObajGraczeNieMogąWykonaćRuchu:
                    MessageBox.Show("Obaj gracze nie mogą wykonać ruchu");
                    koniecGry = true;
                    break;
                case ReversiSilnik.SytuacjaNaPlanszy.WszystkiePolaPlanszyZajęte:
                    koniecGry = true;
                    break;
            }

            if(koniecGry)
            {
                int numerZwycięzcy = silnik.NumerGraczaMającegoPrzewagę;
                if (numerZwycięzcy != 0) MessageBox.Show("Wygrał gracz " + silnik.NumerGraczaMającegoPrzewagę);
                else MessageBox.Show("Remis");

                silnik = new ReversiSilnik(silnik.NumerGraczaWykonującegoNastępnyRuch);
                twórzPlanszę();
                pokażPlanszę();
                lbRuchyGracz1.Items.Clear();
                lbRuchyGracz2.Items.Clear();
            }
        }        

        private void pokażPlanszę()
        {
            for (int poziomo = 0; poziomo < planszaWidok.GetLength(0); ++poziomo)
                for (int pionowo = 0; pionowo < planszaWidok.GetLength(1); ++pionowo)
                {
                    planszaWidok[poziomo, pionowo].BackColor = 
                        koloryPól[silnik.PobierzStanPola(poziomo, pionowo)];                    
                }

            btnKolorGraczaWykonującegoRuch.BackColor =
                        koloryPól[silnik.NumerGraczaWykonującegoNastępnyRuch];
        }

        public Form1()
        {
            InitializeComponent();

            twórzPlanszę();
            pokażPlanszę();
            lbRuchyGracz1.ForeColor = koloryPól[1];
            lbRuchyGracz2.ForeColor = koloryPól[2];

            //int wynik = silnik.PołóżKamień(4, 2);
            //pokażPlanszę();
        }

        private static string symbolPola(int poziomo, int pionowo)
        {
            if (poziomo > 7) throw new ArgumentException("Indeks poziomo za duży");
            return "ABCDEFGH"[poziomo] + pionowo.ToString();
        }
    }
}
