﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Reversi
{
    class ReversiSilnik
    {
        //gracze: 1, 2, 0 - puste

        public int SzerokośćPlanszy { get; private set; }
        public int WysokośćPlanszy { get; private set; }

        private int[,] plansza;
        public int NumerGraczaWykonującegoNastępnyRuch { get; private set; } = 1;

        private static int numerPrzeciwnika(int numerGracza)
        {
            return (numerGracza == 1) ? 2 : 1;
        }

        private bool czyWspółrzędnePolaPrawidłowe(int poziomo, int pionowo)
        {
            return
                poziomo >= 0 && poziomo < SzerokośćPlanszy &&
                pionowo >= 0 && pionowo < WysokośćPlanszy;
        }

        public int PobierzStanPola(int poziomo, int pionowo)
        {
            if (!czyWspółrzędnePolaPrawidłowe(poziomo, pionowo))
                throw new ArgumentException("Nieprawidłowe współrzędne pola");
            return plansza[poziomo, pionowo];
        }

        private void czyśćPlanszę()
        {
            for(int poziomo = 0; poziomo < SzerokośćPlanszy; poziomo++)
                for(int pionowo = 0; pionowo < WysokośćPlanszy; pionowo++)
                {
                    plansza[poziomo, pionowo] = 0;
                }

            int środekSzer = SzerokośćPlanszy / 2;
            int środekWys = WysokośćPlanszy / 2;
            plansza[środekSzer - 1, środekWys - 1] = plansza[środekSzer, środekWys] = 1;
            plansza[środekSzer, środekWys - 1] = plansza[środekSzer - 1, środekWys] = 2;
        }

        public ReversiSilnik(int numerGraczaRozpoczynającego = 1, int szerokośćPlanszy = 8, int wysokośćPlanszy = 8)
        {
            if (numerGraczaRozpoczynającego < 1 || numerGraczaRozpoczynającego > 2)
                throw new ArgumentException("Niepoprawny numer gracza");

            SzerokośćPlanszy = szerokośćPlanszy;
            WysokośćPlanszy = wysokośćPlanszy;
            plansza = new int[SzerokośćPlanszy, WysokośćPlanszy];

            czyśćPlanszę(); //bez sensu

            NumerGraczaWykonującegoNastępnyRuch = numerGraczaRozpoczynającego;
        }

        public int PołóżKamień(int poziomo, int pionowo, bool tylkoTestuj = false) //-1 - nie położył kamienia
        {
            if (!czyWspółrzędnePolaPrawidłowe(poziomo, pionowo))
                throw new Exception("Nieprawidłowe współrzędne pola planszy");

            if (plansza[poziomo, pionowo] != 0)
                //return -1;
                throw new Exception("Pole nie jest puste");

            List<Tuple<int,int>> przejmowanePola = new List<Tuple<int, int>>();

            for (int kierunekPoziomo = -1; kierunekPoziomo <= 1; kierunekPoziomo++)
                for (int kierunekPionowo = -1; kierunekPionowo <= 1; kierunekPionowo++)
                {
                    if (kierunekPoziomo == 0 && kierunekPionowo == 0) continue;

                    List<Tuple<int, int>> przejmowanePolaWKierunku = new List<Tuple<int, int>>();                    

                    int i = poziomo;
                    int j = pionowo;
                    //bool znalezionyKamieńPrzeciwnika = false;
                    bool znalezionyKamieńGraczaWykonująceRuch = false;
                    bool znalezionePustePole = false;
                    bool osiągniętaKrawędźPlanszy = false;
                    do
                    {
                        i += kierunekPoziomo;
                        j += kierunekPionowo;
                        if (!czyWspółrzędnePolaPrawidłowe(i, j))
                        {
                            osiągniętaKrawędźPlanszy = true;
                            break;
                        }

                        if (plansza[i, j] == NumerGraczaWykonującegoNastępnyRuch)
                            znalezionyKamieńGraczaWykonująceRuch = true;
                        if (plansza[i, j] == 0) znalezionePustePole = true;
                        if (plansza[i, j] == numerPrzeciwnika(NumerGraczaWykonującegoNastępnyRuch))
                        {
                            przejmowanePolaWKierunku.Add(new Tuple<int, int>(i, j));
                            //znalezionyKamieńPrzeciwnika = true;
                        }
                    }
                    while (!znalezionyKamieńGraczaWykonująceRuch && !znalezionePustePole && !osiągniętaKrawędźPlanszy);

                    if (!znalezionePustePole && !osiągniętaKrawędźPlanszy) przejmowanePola.AddRange(przejmowanePolaWKierunku);
                }

            if (przejmowanePola.Count == 0) return 0;
            else
            {
                if (!tylkoTestuj)
                {
                    plansza[poziomo, pionowo] = NumerGraczaWykonującegoNastępnyRuch;
                    foreach (Tuple<int, int> pole in przejmowanePola)
                        plansza[pole.Item1, pole.Item2] = NumerGraczaWykonującegoNastępnyRuch;
                    zmieńBieżącegoGracza();
                }
                return przejmowanePola.Count;
            }            
        }

        private void zmieńBieżącegoGracza()
        {
            NumerGraczaWykonującegoNastępnyRuch = numerPrzeciwnika(NumerGraczaWykonującegoNastępnyRuch);
        }

        private bool czyBieżącyGraczMożeWykonaćRuch()
        {
            int liczbaPólPoprawnych = 0;
            for(int i = 0; i < SzerokośćPlanszy; ++i)
                for(int j = 0; j < WysokośćPlanszy; ++j)
                    if(plansza[i,j] == 0 && PołóżKamień(i, j, true) > 0)
                        liczbaPólPoprawnych++;
            return liczbaPólPoprawnych > 0;
        }

        public void Pasuj()
        {
            if (czyBieżącyGraczMożeWykonaćRuch()) throw new Exception("Nie możesz sobie odpuścić ruchy");
            zmieńBieżącegoGracza();
        }

        public enum SytuacjaNaPlanszy
        {
            RuchJestMożliwy,
            BieżącyGraczNieMożeWykonaćRuchu,
            ObajGraczeNieMogąWykonaćRuchu,
            WszystkiePolaPlanszyZajęte
        }

        private int liczPola(int wartośćWzorcowa)
        {
            int liczbaPól = 0;
            for (int i = 0; i < SzerokośćPlanszy; ++i)
                for (int j = 0; j < WysokośćPlanszy; ++j)
                    if (plansza[i, j] == wartośćWzorcowa)
                        liczbaPól++;
            return liczbaPól;
        }

        private int liczbaPustychPól
        {
            get
            {
                return liczPola(0);
            }
        }

        private int liczbaPólGracza1
        {
            get
            {
                return liczPola(1);
            }
        }

        private int liczbaPólGracza2
        {
            get
            {
                return liczPola(2);
            }
        }

        public int NumerGraczaMającegoPrzewagę
        {
            get
            {
                if (liczbaPólGracza1 == liczbaPólGracza2) return 0;
                else return (liczbaPólGracza1 > liczbaPólGracza2) ? 1 : 2;
            }
        }

        public SytuacjaNaPlanszy ZbadajSytuacjęNaPlanszy() //czemu publiczna???????????
        {
            if (liczbaPustychPól == 0) return SytuacjaNaPlanszy.WszystkiePolaPlanszyZajęte;

            bool czyMożliwyRuch = czyBieżącyGraczMożeWykonaćRuch();
            if (czyMożliwyRuch) return SytuacjaNaPlanszy.RuchJestMożliwy;
            else
            {
                zmieńBieżącegoGracza();
                bool czyOponentMożeWykonaćRuch = czyBieżącyGraczMożeWykonaćRuch();
                zmieńBieżącegoGracza();
                if (czyOponentMożeWykonaćRuch) return SytuacjaNaPlanszy.BieżącyGraczNieMożeWykonaćRuchu;
                else return SytuacjaNaPlanszy.ObajGraczeNieMogąWykonaćRuchu;
            }
        }


    }
}
